﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using HugeFlow.Surface.Helpers;

namespace LongClickButtonTest
{
    /// <summary>
    /// 원형모양의 눌린 시간을 보여주는 컨트롤
    /// </summary>
    [TemplatePart(Name = DialPercentageIndicator.MovingTipPanelName, Type = typeof(Panel))]
    [TemplatePart(Name = DialPercentageIndicator.DialPanelName, Type = typeof(Panel))]
    [TemplatePart(Name = DialPercentageIndicator.TestPanelName, Type = typeof(Panel))]
    [TemplatePart(Name = DialPercentageIndicator.PressImageButtonName, Type = typeof(Image))]
    public class DialPercentageIndicator : Control
    {
        internal const string MovingTipPanelName = "MovingTipPanel";
        internal Panel MovingTipPanel;

        internal const string DialPanelName = "DialPanel";
        internal Panel DialPanel;

        internal const string TestPanelName = "TestPanel";
        internal Panel TestPanel;

        internal const string PressImageButtonName = "PressButtonImage";
        internal Image PressImage;

        private RotateTransform _MovingTipRotateTransform;
        private TranslateTransform _MovingTipTranslateTransform;

        #region Percentage

        /// <summary>
        ///
        /// </summary>
        public double Percentage
        {
            get { return (double)GetValue(PercentageProperty); }
            set { SetValue(PercentageProperty, value); }
        }

        public static readonly DependencyProperty PercentageProperty =
                    DependencyProperty.Register(
                          "Percentage",
                          typeof(double),
                          typeof(DialPercentageIndicator),
                          new PropertyMetadata(0.0, OnPercentagePropertyChanged));

        private static void OnPercentagePropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DialPercentageIndicator _DialPercentageIndicator = d as DialPercentageIndicator;
            if (_DialPercentageIndicator != null)
            {
                _DialPercentageIndicator.SetPercentage((double)e.NewValue);
            }
        }

        private void SetPercentage(double percentage)
        {
            percentage = Math.Min(100.0, Math.Max(0.0, percentage % 100.0));

            if (DialPanel != null)
            {
                double angle = 360 * percentage / 100.0;
                if (_MovingTipRotateTransform != null)
                {
                    _MovingTipRotateTransform.Angle = angle;
                }

                if (percentage <= 0)
                {
                    DialPanel.Opacity = 0.0;
                    MovingTipPanel.Visibility = System.Windows.Visibility.Collapsed;
                }
                else if (percentage >= 100.0)
                {
                    DialPanel.Clip = null;
                    DialPanel.Opacity = 1.0;
                    MovingTipPanel.Visibility = System.Windows.Visibility.Visible;
                }
                else
                {
                    DialPanel.Opacity = 1.0;
                    MovingTipPanel.Visibility = System.Windows.Visibility.Visible;

                    var geo = new PathGeometry();
                    PathFigure pf = new PathFigure();
                    geo.Figures.Add(pf);

                    if (DialPanel.ActualWidth == 0.0 ||
                        DialPanel.ActualHeight == 0.0)
                    {
                        return;
                    }

                    Point center = new Point()
                    {
                        X = DialPanel.ActualWidth / 2.0,
                        Y = DialPanel.ActualHeight / 2.0,
                    };

                    Point startPoint = new Point()
                    {
                        X = center.X,
                        Y = 0,
                    };

                    double radiusX = center.X;
                    double radiusY = center.Y;
                    double radius = Math.Max(radiusX, radiusY);
                    double endAngle = angle - 90.0;
                    double angleInRadian = MathHelper.DegreeToRadian(endAngle);

                    double cosValue = Math.Cos(angleInRadian);
                    double sinValue = Math.Sin(angleInRadian);

                    Point endPoint = new Point()
                    {
                        X = center.X + radius * cosValue,
                        Y = center.Y + radius * sinValue,
                    };

                    pf.StartPoint = center;

                    // 첫째 직선
                    LineSegment ls1 = new LineSegment() { Point = startPoint };
                    pf.Segments.Add(ls1);

                    // 두번째 호
                    ArcSegment as1 = new ArcSegment()
                    {
                        IsLargeArc = angle > 180.0,
                        Point = endPoint,
                        SweepDirection = SweepDirection.Clockwise,
                    };

                    //if (angle <= 90.0)
                    //{
                    //    as1.Size = new Size()
                    //    {
                    //        Width = endPoint.X - center.X,
                    //        Height = radius,
                    //    };
                    //}
                    //else if (angle <= 180.0)
                    //{
                    //    as1.Size = new Size()
                    //    {
                    //        Width = DialPanel.ActualWidth - center.X,
                    //        Height = endPoint.Y
                    //    };
                    //}
                    //else if (angle <= 270.0)
                    //{
                    //    as1.Size = new Size()
                    //    {
                    //        Width = DialPanel.ActualWidth - center.X,
                    //        Height = DialPanel.ActualHeight
                    //    };
                    //}
                    //else
                    //{
                    //    as1.Size = new Size()
                    //    {
                    //        Width = DialPanel.ActualWidth,
                    //        Height = DialPanel.ActualHeight
                    //    };
                    //}

                    as1.Size = new Size(radiusX, radiusY);

                    pf.Segments.Add(as1);

                    // 세번째 직선
                    LineSegment ls2 = new LineSegment() { Point = center };
                    pf.Segments.Add(ls2);

                    DialPanel.Clip = geo;

                    //Rectangle rect = new Rectangle()
                    //{
                    //    Width = as1.Size.Width,
                    //    Height = as1.Size.Height,
                    //    Opacity = 0.9,
                    //    Fill = new SolidColorBrush(Colors.SkyBlue),
                    //    VerticalAlignment = System.Windows.VerticalAlignment.Top,
                    //    HorizontalAlignment = System.Windows.HorizontalAlignment.Right,
                    //};

                    //Path a = new Path()
                    //{
                    //    Data = geo,
                    //    Fill = new SolidColorBrush(Colors.Pink),
                    //    Opacity = 0.9,
                    //};

                    //TestPanel.Children.Clear();
                    //TestPanel.Children.Add(rect);
                    //TestPanel.Children.Add(a);
                }
            }
        }

        #endregion Percentage

        public DialPercentageIndicator()
        {
            this.DefaultStyleKey = typeof(DialPercentageIndicator);
            this.SizeChanged += new SizeChangedEventHandler(DialPercentageIndicator_SizeChanged);
            //this.MouseLeftButtonUp += new System.Windows.Input.MouseButtonEventHandler(DialPercentageIndicator_MouseLeftButtonUp);
        }

        private void DialPercentageIndicator_MouseLeftButtonUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            this.Percentage += 5;
        }

        private void DialPercentageIndicator_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            SetPercentage(this.Percentage);
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            MovingTipPanel = this.GetTemplateChild(DialPercentageIndicator.MovingTipPanelName) as Panel;
            if (MovingTipPanel != null)
            {
                MovingTipPanel.RenderTransformOrigin = new Point(0.5, 0.5);
                _MovingTipRotateTransform = new RotateTransform();
                _MovingTipTranslateTransform = new TranslateTransform();

                var tGroup = new TransformGroup();
                tGroup.Children.Add(_MovingTipRotateTransform);
                tGroup.Children.Add(_MovingTipTranslateTransform);

                MovingTipPanel.RenderTransform = tGroup;
            }

            DialPanel = this.GetTemplateChild(DialPercentageIndicator.DialPanelName) as Panel;
            if (DialPanel != null)
            {
            }

            TestPanel = this.GetTemplateChild(DialPercentageIndicator.TestPanelName) as Panel;
            PressImage = this.GetTemplateChild(DialPercentageIndicator.PressImageButtonName) as Image;
            this.SetPercentage(this.Percentage);
        }
    }
}