﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Microsoft.Surface;

namespace LongClickButtonTest
{
    [TemplatePart(Name = LongClickButton.DialName, Type = typeof(DialPercentageIndicator))]
    public class LongClickButton : Control
    {
        internal const string DialName = "Dial";
        internal DialPercentageIndicator Dial;

        #region ClickDuration

        /// <summary>
        /// 눌렸음이 인정되는 시간 (ms)
        /// </summary>
        public int ClickDuration
        {
            get { return (int)GetValue(ClickDurationProperty); }
            set { SetValue(ClickDurationProperty, value); }
        }

        public static readonly DependencyProperty ClickDurationProperty =
                    DependencyProperty.Register(
                          "ClickDuration",
                          typeof(int),
                          typeof(LongClickButton),
                          new PropertyMetadata(2500, OnClickDurationPropertyChanged));

        private static void OnClickDurationPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            LongClickButton _LongClickButton = d as LongClickButton;
            if (_LongClickButton != null)
            {
            }
        }

        #endregion ClickDuration

        private int _TouchId = 0;
        private bool _IsHolding = false;
        private int _StartTick = 0;

        public LongClickButton()
        {
            this.DefaultStyleKey = typeof(LongClickButton);

            this.MouseLeftButtonDown += new System.Windows.Input.MouseButtonEventHandler(LongClickButton_MouseLeftButtonDown);
            this.MouseLeftButtonUp += new System.Windows.Input.MouseButtonEventHandler(LongClickButton_MouseLeftButtonUp);
            this.TouchDown += new EventHandler<System.Windows.Input.TouchEventArgs>(LongClickButton_TouchDown);
            this.TouchUp += new EventHandler<System.Windows.Input.TouchEventArgs>(LongClickButton_TouchUp);
        }

        private void CompositionTarget_Rendering(object sender, EventArgs e)
        {
            if (_IsHolding == false)
                return;

            if (ClickDuration == 0.0)
                return;

            var duration = Environment.TickCount - _StartTick;
            var percent = Math.Min(100.0, (double)duration / ClickDuration * 100.0);

            if (Dial != null)
            {
                Dial.Percentage = percent;
                if (percent == 100.0)
                {
                    CompositionTarget.Rendering -= new EventHandler(CompositionTarget_Rendering);
                    OnClick();

                    this.Init();
                }
            }
        }

        private void Init()
        {
            _StartTick = 0;
            Dial.Percentage = 0.0;
            _TouchId = 0;
        }

        private void LongClickButton_TouchUp(object sender, System.Windows.Input.TouchEventArgs e)
        {
            if (_IsMouseDown == true)
                return;

            var tp = e.GetTouchPoint(null);
            if (_TouchId != tp.TouchDevice.Id)
                return;

            _TouchId = 0;
            _IsHolding = false;
            if (Dial != null)
            {
                Dial.Percentage = 0.0;
            }

            this.ReleaseTouchCapture(tp.TouchDevice);
        }

        private void LongClickButton_MouseLeftButtonUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            _IsMouseDown = false;
            _IsHolding = false;
            if (Dial != null)
            {
                Dial.Percentage = 0.0;
            }
            Dial.PressImage.Visibility = System.Windows.Visibility.Collapsed;
            this.ReleaseMouseCapture();
        }

        private void LongClickButton_TouchDown(object sender, System.Windows.Input.TouchEventArgs e)
        {
            if (_IsMouseDown == true)
                return;

            var tp = e.GetTouchPoint(null);
            if (_TouchId != 0)
                return;

            _TouchId = tp.TouchDevice.Id;
            _IsHolding = true;
            _StartTick = Environment.TickCount;

            if (Dial != null)
            {
                CompositionTarget.Rendering -= new EventHandler(CompositionTarget_Rendering);
                CompositionTarget.Rendering += new EventHandler(CompositionTarget_Rendering);
            }

            this.CaptureTouch(tp.TouchDevice);
        }

        private bool _IsMouseDown = false;

        private void LongClickButton_MouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            _IsMouseDown = true;
            _IsHolding = true;
            _StartTick = Environment.TickCount;

            if (Dial != null)
            {
                CompositionTarget.Rendering -= new EventHandler(CompositionTarget_Rendering);
                CompositionTarget.Rendering += new EventHandler(CompositionTarget_Rendering);
            }

            Dial.PressImage.Visibility = System.Windows.Visibility.Visible;
            this.CaptureMouse();
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            Dial = this.GetTemplateChild(LongClickButton.DialName) as DialPercentageIndicator;
        }

        #region Click

        public event EventHandler Click;

        private void OnClick()
        {
            if (Click == null)
                return;

            Click(this, null);
        }

        #endregion Click
    }
}